import 'dart:async';

import 'package:audioplayers/audioplayers.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:multimedia_gallery/audio/widget/image_background_gradient.dart';

AudioPlayer player = AudioPlayer();

urlFile(String path) {
  return player.play(UrlSource(path));
}

assetFile(String path) {
  return player.play(AssetSource(path));
}

localFile(String path) {
  return player.play(DeviceFileSource(path));
}

class AudioViewer extends StatefulWidget {
  final ImageProvider? image;
  final String? songName;
  final String? artistName;
  final EdgeInsets? screenPadding;
  final double? imageWidth;
  final BorderRadius? imageBorder;
  final BoxFit? imageFit;
  final TextStyle? songNameTextstyle;
  final TextStyle? artistNameTextstyle;
  final SliderThemeData? sliderTheme;
  final Icon? icon;
  final ButtonStyle? iconStyle;
  final void Function()? onPressed;
  final Future<void>? path;

  const AudioViewer(
      {Key? key,
      required this.path,
      required this.image,
      required this.songName,
      required this.artistName,
      this.screenPadding,
      this.imageBorder,
      this.imageWidth,
      this.imageFit,
      this.artistNameTextstyle,
      this.songNameTextstyle,
      this.sliderTheme,
      this.icon,
      this.iconStyle,
      this.onPressed})
      : super(key: key);

  @override
  State<AudioViewer> createState() => _AudioViewerState();
}

class _AudioViewerState extends State<AudioViewer> {
  late ScrollController _titleController;
  late ScrollController _artistController;
  Duration? audioTimestamp;
  Duration? audioDuration;
  PlayerState? state;
  double? sliderPosition;
  double? sliderMaxPosition;

  StreamSubscription? durationSubscription;
  StreamSubscription? positionSubscription;
  StreamSubscription? playerCompleteSubscription;
  StreamSubscription? playerStateChangeSubscription;

  @override
  void initState() {
    super.initState();
    _titleController = ScrollController();
    _artistController = ScrollController();
    _startAutoScroll(_titleController);
    _startAutoScroll(_artistController);
    widget.path;
    getDuration();
    getCurrentTimestamp();
    playerController();
  }

  @override
  void dispose() {
    _titleController.dispose();
    _artistController.dispose();
    durationSubscription?.cancel();
    positionSubscription?.cancel();
    playerCompleteSubscription?.cancel();
    playerStateChangeSubscription?.cancel();
    player.dispose();
    super.dispose();
  }

  void _startAutoScroll(ScrollController controller) {
    Timer.periodic(const Duration(seconds: 10), (_) {
      if (controller.hasClients) {
        double maxScrollExtent = controller.position.maxScrollExtent;
        double minScrollExtent = controller.position.minScrollExtent;
        if (controller.offset != maxScrollExtent) {
          controller.animateTo(maxScrollExtent,
              duration: const Duration(seconds: 5), curve: Curves.linear);
        } else {
          controller.animateTo(minScrollExtent,
              duration: const Duration(seconds: 5), curve: Curves.linear);
        }
      }
    });
  }

  void resumeAudio() {
    player.resume();
  }

  void pauseAudio() {
    player.pause();
  }

  void getDuration() {
    player.getDuration().then((value) => setState(() => audioDuration = value));
  }

  void getCurrentTimestamp() {
    player
        .getCurrentPosition()
        .then((value) => setState(() => audioTimestamp = value));
  }

  void playerController() {
    durationSubscription = player.onDurationChanged.listen((d) {
      setState(() {
        audioDuration = d;
        sliderMaxPosition = d.inMilliseconds.toDouble();
      });
    });

    positionSubscription = player.onPositionChanged.listen(
      (p) => setState(() {
        audioTimestamp = p;
        sliderPosition = p.inMilliseconds.toDouble();
      }),
    );

    playerCompleteSubscription = player.onPlayerComplete.listen((event) {
      setState(() {
        state = PlayerState.stopped;
      });
    });

    playerStateChangeSubscription = player.onPlayerStateChanged.listen((state) {
      setState(() {
        state = state;
      });
    });
  }

  void seekTo(double value) {
    player.seek(Duration(milliseconds: value.round()));
  }

  String formatDuration(Duration duration) {
    String twoDigits(int n) => n.toString().padLeft(2, '0');
    String minutes = duration.inMinutes.remainder(60).toString();
    String twoDigitSeconds = twoDigits(duration.inSeconds.remainder(60));
    return '$minutes:$twoDigitSeconds';
  }

  @override
  Widget build(BuildContext context) {
    SystemChrome.setPreferredOrientations([
      DeviceOrientation.portraitUp,
      DeviceOrientation.portraitDown,
    ]);
    return widget.image.toString().contains('.gif')
        ? Container(
            decoration: BoxDecoration(
                image: DecorationImage(
                    image: widget.image ?? const AssetImage(''),
                    fit: BoxFit.fitHeight)),
            child: Stack(
              children: [Container(color: Colors.black38), buildContent()],
            ),
          )
        : GradientBackground(
            image: widget.image ?? const AssetImage(''), child: buildContent());
  }

  Widget buildContent() {
    return Scaffold(
        backgroundColor: Colors.transparent,
        appBar: AppBar(
            backgroundColor: Colors.transparent,
            leading: IconButton(
                icon: const Icon(Icons.arrow_back),
                onPressed: () => Navigator.pop(context),
                color: Colors.white)),
        body: Container(
            padding: widget.screenPadding ?? const EdgeInsets.all(12),
            child:
                Column(mainAxisAlignment: MainAxisAlignment.center, children: [
              widget.image.toString().contains('.gif')
                  ? Expanded(
                      flex: 2,
                      child: SizedBox(
                          height: MediaQuery.of(context).size.height / 1.8))
                  : Expanded(
                      flex: 2,
                      child: ClipRRect(
                          borderRadius:
                              widget.imageBorder ?? BorderRadius.circular(10),
                          child: Image(
                              image: widget.image ?? const AssetImage(''),
                              fit: widget.imageFit ?? BoxFit.scaleDown,
                              width: widget.imageWidth ??
                                  MediaQuery.of(context).size.width / 1.5))),
              Flexible(
                  child: Column(children: [
                SingleChildScrollView(
                    scrollDirection: Axis.horizontal,
                    controller: _titleController,
                    child: Column(mainAxisSize: MainAxisSize.min, children: [
                      Flexible(
                          child: FittedBox(
                              fit: BoxFit.scaleDown,
                              child: Text(widget.songName ?? '',
                                  style: widget.songNameTextstyle ??
                                      const TextStyle(
                                          fontSize: 20,
                                          fontWeight: FontWeight.bold,
                                          color: Colors.white))))
                    ])),
                SingleChildScrollView(
                    scrollDirection: Axis.horizontal,
                    controller: _artistController,
                    child: Column(mainAxisSize: MainAxisSize.min, children: [
                      Flexible(
                          child: FittedBox(
                              fit: BoxFit.scaleDown,
                              child: Text(widget.artistName ?? '',
                                  style: widget.artistNameTextstyle ??
                                      const TextStyle(
                                          fontSize: 20,
                                          fontWeight: FontWeight.w600,
                                          color: Colors.white70))))
                    ])),
                SliderTheme(
                    data: widget.sliderTheme ??
                        SliderThemeData(
                            overlayShape: SliderComponentShape.noOverlay,
                            thumbColor: Colors.white,
                            activeTrackColor: Colors.white,
                            inactiveTrackColor: Colors.black26),
                    child: Slider(
                      value: sliderPosition ?? 0,
                      min: 0,
                      max: sliderMaxPosition ?? 0,
                      onChanged: (value) {
                        sliderPosition = value;
                      },
                      onChangeEnd: (value) {
                        seekTo(value);
                      },
                    )),
                Padding(
                    padding: const EdgeInsets.all(10.0),
                    child: Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: [
                          Text(formatDuration(audioTimestamp ?? Duration.zero),
                              style: const TextStyle(
                                  fontSize: 14,
                                  fontWeight: FontWeight.bold,
                                  color: Colors.white)),
                          Text(
                              formatDuration((audioDuration ?? Duration.zero) -
                                  (audioTimestamp ?? Duration.zero)),
                              style: const TextStyle(
                                  fontSize: 14,
                                  fontWeight: FontWeight.bold,
                                  color: Colors.white))
                        ])),
                IconButton.filled(
                    onPressed: widget.onPressed ??
                        () {
                          setState(() {
                            (player.state.name == 'playing')
                                ? pauseAudio()
                                : resumeAudio();
                          });
                        },
                    icon: widget.icon ??
                        Icon(player.state.name == 'playing'
                            ? Icons.pause
                            : Icons.play_arrow),
                    style: widget.iconStyle ??
                        const ButtonStyle(
                            backgroundColor:
                                MaterialStatePropertyAll<Color>(Colors.white),
                            iconColor:
                                MaterialStatePropertyAll<Color>(Colors.black),
                            iconSize: MaterialStatePropertyAll(20)))
              ])),
            ])));
  }
}
